package com.supermap.imobile.mdatacollector;

/**
 * <p>
 * Title: Sample code of multimedia acquisition
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is for SuperMap iMobile for Android Sample Code 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ---------------------SuperMap iMobile for Android Sample Code------------------------
 * 
 * 1. Introduction: How to conduct the multimedia acquisition and data upload and download
 * 2. Sample data: /sdcard/SampleData/MDataCollectorData/media.smwu
 * 3. Members: 
 *		MDataCollector.captureImage();
 *      MDataCollector.captureVideo();
 *      MDataCollector.startCaptureAudio();
 *      MDataCollector.stopCaptureAudio();
 *      MDataCollector.setLocalFilePath();
 *      MDataCollector.setMediaDataset();
 *      MDataCollector.downloadMediaFile();
 *      MDataCollector.uploadMediaFile();
 *      MDataCollector.addNetworkAccessMediaFileListener();
 * 4. Steps:
 *   (1)Click the Collect button. You can take picture, video, and record;
 *   (2)Click View to view the collected points;
 *   (3)Click the Network, and you can upload and downoad. Before that, you should login the iPortal.
 *
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.supermap.data.Color;
import com.supermap.data.Datasource;
import com.supermap.data.Environment;
import com.supermap.data.GeoRegion;
import com.supermap.data.GeoStyle;
import com.supermap.data.GeometryType;
import com.supermap.data.Size2D;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Action;
import com.supermap.mapping.Layer;
import com.supermap.mapping.LayerSettingVector;
import com.supermap.mapping.Map;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;

public class MainActivity extends Activity  {

	private Button btn_Network      = null;
	private Button btn_captureImage = null;
	private Button btn_captureVideo = null;
	private Button btn_captureAudio = null;
	private Button btn_download     = null;
	private Button btn_upload       = null;
	private Button btn_stopCaptureAudio = null;
	private TextView mTextView      = null;
	private boolean mExitEnable = false;
	
	private MyApplication  m_App = null;
	private LoginPopUp     m_LoginPopup = null;
	private NetworkAccess  m_NetworkAccess = null;
	private MapView        m_MapView    = null;
	private MapControl     m_MapControl = null;
	private Map            m_Map        = null;
	private Workspace      m_Workspace  = null;
	private Datasource     m_Datasource = null;
	private Layer          m_MediaLayer = null;
	private DrawRegion     m_DrawRegion = null;
	
	private char transmission = 0;          // 1:download, 2:upload
	
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Environment.initialization(this);
        setContentView(R.layout.activity_main);
        m_App = MyApplication.getInstance();
        m_App.registerActivity(this);
        
       	 if(initMap()) {
       		m_Map.setFullScreenDrawModel(true);                    // Set after the map.open()
        	initView();
        	m_NetworkAccess = new NetworkAccess(this,m_Datasource, m_Map);
			m_LoginPopup = new LoginPopUp(this, m_NetworkAccess);
			m_NetworkAccess.setLoginPopUp(m_LoginPopup);
			m_DrawRegion = new DrawRegion(m_MapControl, m_NetworkAccess);
        } else {
        	m_App.showInfo("Failed to open the map");
        }
    }
	
    /**
     * Initialize the map
     * @return  true or false, return true if open; otherwise, false;
     */
    private boolean initMap() {
    	m_MapView = (MapView) findViewById(R.id.m_MapView);
    	m_MapControl = m_MapView.getMapControl();
    	m_Map = m_MapControl.getMap();
    	
    	m_Workspace = new Workspace();
    	String pathWK = MyApplication.RootPath + "/SampleData/MDataCollectorData/media.smwu";
    	WorkspaceConnectionInfo wkInfo = new WorkspaceConnectionInfo();
    	wkInfo.setServer(pathWK);
    	wkInfo.setType(WorkspaceType.SMWU);
    	m_Workspace.open(wkInfo);
    	m_Map.setWorkspace(m_Workspace);
    	m_Datasource = m_Workspace.getDatasources().get("media");
    	if(m_Datasource == null)
    		return false;
    	return m_Map.open(m_Workspace.getMaps().get(0));
    }
    
    /**
     * Initialize the interface button
     */
    private void initView() {
    
    	((Button) findViewById(R.id.btn_collect)).setOnClickListener(listener1);
    	((Button) findViewById(R.id.btn_review)).setOnClickListener(listener1);
    	btn_Network = (Button) findViewById(R.id.btn_network);
    	btn_Network.setOnClickListener(listener1);
    	
    	btn_captureImage = (Button) findViewById(R.id.btn_captureImage);
    	btn_captureVideo = (Button) findViewById(R.id.btn_captureVideo);
    	btn_captureAudio = (Button) findViewById(R.id.btn_captureAudio);
    	btn_download     = (Button) findViewById(R.id.btn_download);
    	btn_upload       = (Button) findViewById(R.id.btn_upload);
    	btn_stopCaptureAudio = (Button) findViewById(R.id.btn_stopcaptureaudio);
    	
     	btn_captureImage.setOnClickListener(listener2);
    	btn_captureVideo.setOnClickListener(listener2);
    	btn_captureAudio.setOnClickListener(listener2);
    	btn_download.setOnClickListener(listener2);
    	btn_upload.setOnClickListener(listener2);
    	btn_stopCaptureAudio.setOnClickListener(listener2);
    }
    
	private OnClickListener listener1 = new OnClickListener() {

		@Override
		public void onClick(View v) {
			// TODO Auto-generated method stub
			switch(v.getId()){
			case R.id.btn_collect:
				if (!m_NetworkAccess.isMediaDatasetExisted()) {
					
					m_NetworkAccess.initMDataCollector();
					
				} 
				if(btn_captureVideo.getVisibility() == View.VISIBLE){
					setButtonVisibility1(View.GONE);
				}else{
				    setButtonVisibility1(View.VISIBLE);
				}
				if(m_MapControl.getAction() == Action.DRAWPLOYGON)
				   m_DrawRegion.enabledDrawing(false);
				break;
			case R.id.btn_review:
				if (!m_NetworkAccess.isMediaDatasetExisted()) {
					
					m_NetworkAccess.initMDataCollector();
					
				} 
				showMediaData();
				if(m_MapControl.getAction() == Action.DRAWPLOYGON)
					   m_DrawRegion.enabledDrawing(false);
				break;
			case R.id.btn_network:
				if (m_NetworkAccess.isLogined()) {
					if(btn_upload.getVisibility() == View.VISIBLE){
						setButtonVisibility2(View.GONE);
					}else{
					    setButtonVisibility2(View.VISIBLE);
					}
				} else {
					m_LoginPopup.show();
					m_App.showInfo("Please login first");
				}
				if(m_MapControl.getAction() == Action.DRAWPLOYGON)
					   m_DrawRegion.enabledDrawing(false);
				break;
		    default:
			    break;
			}
		}

		private void showMediaData() {
			if (m_MediaLayer == null && m_NetworkAccess.getMDataCollector() != null) {
				m_MediaLayer = m_Map.getLayers().add(m_NetworkAccess.getMDataCollector().getMediaDataset(), true);
				LayerSettingVector layerSetting = new LayerSettingVector();
				GeoStyle geoStyle_P = new GeoStyle();
				geoStyle_P.setLineColor(new Color(255, 50, 20));
				geoStyle_P.setMarkerAngle(14.0);
				geoStyle_P.setMarkerSize(new Size2D(10, 10));
				layerSetting.setStyle(geoStyle_P);
				m_MediaLayer.setAdditionalSetting(layerSetting);
			}
			m_MapControl.setAction(Action.SELECT);
			m_Map.refresh();
		}
	};
	
	private OnClickListener listener2 = new OnClickListener() {

		@Override
		public void onClick(View v) {
			// TODO Auto-generated method stub
			switch(v.getId()){
			case R.id.btn_captureImage:
				m_NetworkAccess.getMDataCollector().captureImage();
				setButtonVisibility1(View.GONE);
				break;
			case R.id.btn_captureVideo:
				m_NetworkAccess.getMDataCollector().captureVideo();
				setButtonVisibility1(View.GONE);
				break;
			case R.id.btn_captureAudio:
				m_NetworkAccess.getMDataCollector().startCaptureAudio();
				setButtonVisibility1(View.GONE);
				btn_stopCaptureAudio.setVisibility(View.VISIBLE);
				break;
			case R.id.btn_download:
				transmission = 1;
				m_DrawRegion.enabledDrawing(true);
				m_App.showInfo("Please draw a region");
				setButtonVisibility2(View.GONE);
				break;
			case R.id.btn_upload:
				transmission = 2;
				m_DrawRegion.enabledDrawing(true);
				setButtonVisibility2(View.GONE);
				m_App.showInfo("Please draw a region");
				break;
			case R.id.btn_stopcaptureaudio:
				m_NetworkAccess.getMDataCollector().stopCaptureAudio();
				btn_stopCaptureAudio.setVisibility(View.GONE);
				break;
			default:
				break;
			}
		}
	};
	
	private void setButtonVisibility1(int viewStatus){
		btn_captureImage.setVisibility(viewStatus);
		btn_captureVideo.setVisibility(viewStatus);
		btn_captureAudio.setVisibility(viewStatus);
	}
	
	private void setButtonVisibility2(int viewStatus){
		btn_download.setVisibility(viewStatus);
		btn_upload.setVisibility(viewStatus);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if(m_NetworkAccess != null && m_NetworkAccess.getMDataCollector() != null)
		   m_NetworkAccess.getMDataCollector().onActivityResult(requestCode, resultCode, data);
    }

	public void setNetworkStatus(boolean enabled, String info){
		btn_Network.setEnabled(enabled);
		if(mTextView == null)
			mTextView = (TextView) findViewById(R.id.tv_info);
		mTextView.setText(info);
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if(keyCode == KeyEvent.KEYCODE_BACK){
			if(!mExitEnable){
				Toast.makeText(this, "Press again to exist the program!", Toast.LENGTH_SHORT).show();
				mExitEnable = true;
			}else{
				m_App.exit();
			}
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}
	
	
		@Override
		public boolean dispatchTouchEvent(MotionEvent event) {
			// TODO Auto-generated method stub
			super.dispatchTouchEvent(event);
			mExitEnable = false;                      
			
			int action = event.getAction();
			
			
			if(action == MotionEvent.ACTION_UP){
				GeoRegion geometry = (GeoRegion)m_MapControl.getCurrentGeometry();
				if (geometry != null && !geometry.isEmpty()&& geometry.getType()==GeometryType.GEOREGION) {
					m_DrawRegion.showPopupWn(this, transmission);
					geometry = null;
					return true;
				}
			}
			return true;
		}
	
}
