package com.supermap.imobile.mdatacollector;

import android.app.Activity;
import android.content.Context;

import com.supermap.data.DatasetVector;
import com.supermap.data.Datasource;
import com.supermap.data.Rectangle2D;
import com.supermap.mapping.Map;
import com.supermap.mdatacollector.MDataCollector;
import com.supermap.mdatacollector.NetworkAccessMediaFileListener;
import com.supermap.services.DataDownloadService;
import com.supermap.services.DataUploadService;
import com.supermap.services.FeatureSet;
import com.supermap.services.ResponseCallback;

public class NetworkAccess {

	private MDataCollector m_MDataCollector;
	private Datasource m_Datasource;
	private MyApplication m_App;
	private Activity m_Activity;
	private DataDownloadService m_DownloadService;
	private DataUploadService m_UploadService;
	private Map m_Map;
	private Rectangle2D regionBounds;
	private LoginPopUp m_LoginPopUp;
	
	private String iServerUrl = "";                   // iServer service address
	private String dvName = "media";                  // Name of point dataset
	private String serviceName = "data-media";       // iServer service names
	private String datasourceName = "media";          // iServer datasource name
	private String urlDataset;                        // Dataset address
	private String iPortalUrl;
	private String UserName;
	private String UserPassword;
	
	private Context m_context;
	private boolean isLogined = false;
	private boolean isLogining = false;
	private boolean isDownloading = false;
	private boolean isUploaded = false;
	private boolean isExisted  = false;
	
	public NetworkAccess(Context context, Datasource datasource, Map map){
		m_Activity  = (Activity) context;
		m_context = context;
		m_App = MyApplication.getInstance();
		m_Datasource = datasource;
		m_Map = map;
		
		m_DownloadService = new DataDownloadService(iServerUrl);
		m_UploadService = new DataUploadService(iServerUrl);
		
		m_DownloadService.setResponseCallback(serviceCallback);
		m_UploadService.setResponseCallback(serviceCallback);
		
//		initMDataCollector();
		
	}

	public void initMDataCollector() {
		if(m_MDataCollector == null) {
			m_MDataCollector = new MDataCollector(m_context);
		    m_MDataCollector.openGPS();
		    try {
		 
		    	isExisted = m_MDataCollector.setMediaDataset(m_Datasource, dvName);
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		    m_MDataCollector.setLocalFilePath(MyApplication.RootPath+ "/SuperMap/MediaTemp/");
		    m_MDataCollector.addNetworkAccessMediaFileListener(mediaFileListener);
		}
	}
	
	/**
	 * Download the multimedia data
	 */
	private void download(){
		if(isLogining){
			changeNetworkStatus(false, "Updating dataset......");
		}else {
			isDownloading = true;
		}
		if(isExisted){
			m_DownloadService.updateDataset(urlDataset,(DatasetVector) m_Datasource.getDatasets().get(dvName));
		}else{
			m_DownloadService.downloadDataset(urlDataset, m_Datasource);
		}
	}

	private void downloadMediaData() {
		// Update the local dataset
		if (m_MDataCollector != null) {
			new Thread(new Runnable() {

				@Override
				public void run() {
					// TODO Auto-generated method stub
					m_MDataCollector.downloadMediaFiles(regionBounds);
				}
			}).start();
		}
	}

	
	/**
	 * Upload datasets and multimedia data
	 */
	private void upload(){
		isLogining = false;
		// Upload dataset
		if(iServerUrl != null)
		    m_UploadService.commitDataset(urlDataset, (DatasetVector)m_MDataCollector.getMediaDataset());
		
		// Update multimedia data
		if (m_MDataCollector != null) {
			new Thread(new Runnable() {

				@Override
				public void run() {
					// TODO Auto-generated method stub
					m_MDataCollector.uploadMediaFiles(regionBounds);
				}
			}).start();
		}
	}
	
	/**
	 * Login iPortal
	 * @param url            iPortal login address
	 * @param UserName       iPortal username 
	 * @param UserPassword   iPortal user password
	 */
	public void login(final String url, final String userName, final String userPassword){
		iPortalUrl = url;
		UserName = userName;
		UserPassword = userPassword;
		if(!isExisted)
			
		   initMDataCollector();
			
		// Login iPortal
		new Thread(new Runnable(){

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if(UserName == null || UserPassword == null || iPortalUrl == null){
		    	}else {
		    		try{
						isLogined = m_MDataCollector.login(iPortalUrl, UserName, UserPassword);
						showLoginStatus("");
		    		}catch(IllegalStateException e){
		    			e.printStackTrace();
		    			showLoginStatus(": The user name or password is error");
		    		}
		    	}
			}

		}).start();
	}

	
	private void showLoginStatus(final String msg) {
		m_Activity.runOnUiThread(new Runnable(){

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if(isLogined){
					m_LoginPopUp.dismiss();
					m_App.showInfo("Login Success");
					isLogining = true;
				    download();
				}else{
					m_App.showInfo("Failed Login" + msg);
					m_LoginPopUp.setBtnEnable(true);
				}
			}
		});
	}
	
	public void setLoginPopUp(LoginPopUp loginPopUp){
		m_LoginPopUp = loginPopUp;
	}
	
	public boolean isLogined(){
		return isLogined;
	}
	
	public boolean isMediaDatasetExisted(){
		return isExisted;
	}
	
	private void showInfo(final String msg){
		m_Activity.runOnUiThread(new Runnable(){

			@Override
			public void run() {
				// TODO Auto-generated method stub
				m_App.showInfo(msg);
			}
		});
	}
	
	/**
	 * Set the service ip of iServer
	 * @param strIP      ip string of iServer service
	 */
	public void setiServerIpAddr (String strIP){
		iServerUrl = "http://" + strIP + ":8090";
		urlDataset = iServerUrl +"/iserver/services/" + serviceName + "/rest/data/datasources/" + datasourceName +"/datasets/" + dvName;
		m_DownloadService.setUrl(iServerUrl);
		m_UploadService.setUrl(iServerUrl);
	} 
	
	public MDataCollector getMDataCollector(){
		return m_MDataCollector;
	}
	
	private ResponseCallback serviceCallback = new ResponseCallback(){

		@Override
		public void dataServiceFinished(String arg0) {
			// TODO Auto-generated method stub
			
			if(isLogining){
				
				initMDataCollector();
				
				showInfo("Finish updating dataset");
				changeNetworkStatus(true, "");
				isLogining = false;
			}else if(isDownloading){
				downloadMediaData();
				isDownloading = false;
			}else {
				showUploadInfo();
			}
		}

		@Override
		public void receiveResponse(FeatureSet arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void requestFailed(String arg0) {
			// TODO Auto-generated method stub
			
			if(isLogining){
				isLogining = false;
				showInfo("Failed to update dataset:\n" + arg0 );
				changeNetworkStatus(true, "");
			}else if(isDownloading){
				isDownloading = false;
				showInfo("Failed to update dataset:\n" + arg0 );
				changeNetworkStatus(true, "");
			}else {
				if (!str_uploadInfo.equals("")){
					str_uploadInfo = "Failed to upload dataset :\n" + arg0 + "\n" + str_uploadInfo;
				}else{
					str_uploadInfo = "Failed to upload dataset :\n" + arg0;
				}
				showUploadInfo();
			}
		}

		@Override
		public void requestSuccess() {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void addFeatureSuccess(int arg0) {
		}
		
	};
	
	private NetworkAccessMediaFileListener mediaFileListener = new NetworkAccessMediaFileListener(){

		@Override
		public boolean downloadMediaFile(String mediaFileName) {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public boolean uploadMediaFile(String mediaFileName) {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public void onDownloadMediaFiles(String downloadInfo) {
			// TODO Auto-generated method stub
			showInfo("DownloadComplete" + "\n" + downloadInfo);
			changeNetworkStatus(true, "");
		}

		@Override
		public void onUploadMediaFiles(String uploadInfo) {
			// TODO Auto-generated method stub
			if (!str_uploadInfo.equals("")){
				str_uploadInfo = uploadInfo + "\n" + str_uploadInfo;
			}else{
				str_uploadInfo = uploadInfo;
			}
			showUploadInfo();
		}
	};
	
	private String str_uploadInfo = "";
    private void showUploadInfo() {
		if (isUploaded) {
			showInfo("Upload Complete" + "\n" + str_uploadInfo);
			changeNetworkStatus(true, "");
			isUploaded = false;
			str_uploadInfo = "";
		} else {
			isUploaded = true;
		}
	}
    
	private void changeNetworkStatus(final boolean enabled, final String info){
		m_Activity.runOnUiThread(new Runnable(){

			@Override
			public void run() {
				// TODO Auto-generated method stub
				((MainActivity)m_Activity).setNetworkStatus(enabled, info);
			}
		});
	}
	
	public void netWorkTransfering(char transmission, Rectangle2D bounds){
		regionBounds = bounds;
		bounds = m_Map.getBounds();
		if(regionBounds == null){
			showInfo("Drawing area is empty");
		}
		if(1 == transmission){
			download();
			changeNetworkStatus(false, "Downloading......");
		}else if(2 == transmission){
			upload();
			changeNetworkStatus(false, "Uploading......");
		}
	}
}
